@@ -244,6 +244,85 @@ class AddProductsAction(Action):
     name = "Rebuild the Products/ directory ."
     option = '--products'
 
+    def _manMsg(self, msg):
+        """allow easy redirect of manifest messages"""
+        log.info(msg)
+    
+    def _commonLeft(self, common, item):
+        "returns the common string starting left"
+        a, b = common, item
+        c=''
+        if b == None:
+            return a
+        for i in range(len(a)):
+            #run off the end of b?
+            if i>=len(b):
+                return c
+            #always prefer to keep a path if we have one
+            if a.startswith('/') and not b.startswith('/'):
+                return a
+            if a[i] != b[i]:
+                return c
+            c+=a[i]
+        return c
+        
+    def _snipCommon(self, common, s):
+        """snip off the common string from s"""
+        #if there's nothing common - return s
+        if not common:
+            return s
+        #sometimes s is None
+        if not s:
+            return ''
+        # if we snip, say so by putting a +
+        return s.replace(common, '+')
+        
+    MANFORMAT="%26s : %s"
+    MANITEM= "%14s %48s %38s %48s %-20s"
+    cc='='
+    MANLINES=(cc*14, cc*48, cc*38, cc*48, cc*20)
+    MANTITLE=('type', 'source', 'nested source', 'target', 'version')
+    def _printManifest(self, manifest):
+        #show the installation sequence
+        item = manifest['trace'][0]
+        common=[item[ind] for ind in range(len(item))]
+        for item in manifest['trace']:
+            for i in (range(len(item))):
+                c=self._commonLeft(common[i], item[i])
+                common[i:i+1]=[c]
+        self._manMsg( self.MANITEM % self.MANLINES )
+        self._manMsg( self.MANITEM % self.MANTITLE )
+        headings=[]
+        for c in common:
+            if c:
+                headings.append(c+'+')
+            else:
+                headings.append('')
+        self._manMsg( self.MANITEM % tuple(headings) )
+        self._manMsg( self.MANITEM % self.MANLINES )
+        for item in manifest['trace']:
+            it=[self._snipCommon(common[i], item[i]) for i in range(len(item))]
+            it[4:5]=[it[4][0:20]]
+            self._manMsg(self.MANITEM % tuple(it))
+        #show what's actually installed    
+        installed = manifest['installed'].items()
+        installed.sort()
+        self._manMsg('')
+        self._manMsg('-'*170)
+        self._manMsg('Duplicates, and their sources...')
+        self._manMsg('-'*170)
+        dupe=False
+        for product, attempts in installed:
+            if len(attempts) > 1:
+                p=product
+                dupe=True
+                for attempt in attempts:
+                    self._manMsg(self.MANFORMAT % (p, attempt))
+                    p=' '
+        if not dupe:
+            self._manMsg('None found')
+        self._manMsg('='*170)
+                 
     def run(self, options=None):
         target = os.path.join(self.conf.instanceDir(), 'Products')
         try:
@@ -256,14 +335,16 @@ class AddProductsAction(Action):
         os.mkdir(target)
         log.debug("Created it again.")
         log.debug("Adding Products...")
+        manifest={}
         for source in self.conf.sources():
             try:
-                source.addProduct()
+                source.addProduct(manifest)
             except IOError:
                 log.warn('Please fix this.')
-                # XXX Maybe we want to quit with an error here.
+                # XXX Maybe we want to quit with an error here.        
         log.info("Rebuilt the Products/ directory from scratch.")
-
+        if options and options.manifest:
+            self._printManifest(manifest)
 
 class ZopectlAction(ArgumentAction):
     """Action for calling zopectl.
@@ -278,6 +359,45 @@ class ZopectlAction(ArgumentAction):
         actionutils.runZopectl(self.conf, command)
 
 
+class TestAction(ArgumentAction):
+    """Action for calling zopectl with a test command.
+    """
+
+    name = "Runs tests for product PRD (use ALL for all products)."
+    metavar = 'PRD'
+    option = '--test'
+    alternativeOption = '-t'
+
+    def zopeVersion(self):
+        """Return first two numbers of the zope version.
+
+        So '28' for zope 2.8, '29' for zope 2.9.3.
+        """
+
+        version = self.conf.configData['zope_version']
+        version = version.replace('.', '')
+        return version[:2]
+
+    def testCommand(self, product):
+        arguments = []
+        arguments.append('test')
+        version = self.zopeVersion()
+        template_29 = '-s Products.%s'
+        template_27 = '--libdir ' + self.conf.instanceDir() + '/Products/%s'
+        template = template_29
+        if version in ['27', '28']:
+            template = template_27
+        if not product == 'ALL':
+            arguments.append(template % product)
+        command = ' '.join(arguments)
+        return command
+
+    def run(self, options=None):
+        product = options.test
+        command = self.testCommand(product)
+        actionutils.runZopectl(self.conf, command)
+
+
 class ZeoctlAction(ArgumentAction):
     """Action for calling zeoctl.
     """
@@ -315,6 +435,7 @@ class QuickreinstallAction(Action):
 
     def run(self, options=None):
         ploneSiteName = self.conf.configData['plone_site_name']
+        adminUser = self.conf.configData['user']
         if not ploneSiteName:
             log.debug("No plone site name, so no reinstall.")
             log.info("No quickreinstall wanted.")
@@ -325,6 +446,8 @@ class QuickreinstallAction(Action):
         arguments.append(reinstallScript) # The script.
         # First argument: plone site name
         arguments.append(ploneSiteName)
+        # Second argument: id of admin user who creates the plone site
+        arguments.append(adminUser)
         # Next the list of main products
         arguments += self.conf.configData['main_products']
         arguments += self.conf.configData['generic_setup_profiles']
@@ -336,6 +459,70 @@ class QuickreinstallAction(Action):
         log.info("Quick reinstall script called.")
 
 
+class PackAction(Action):
+    """Action that packs your database.
+    """
+
+    name = "Action that packs your database."
+    option = '--pack'
+
+    def scriptLocation(self):
+        """Return the location of the pack script.
+        """
+
+        ourDir = os.path.dirname(os.path.abspath(__file__))
+        location = os.path.join(ourDir, config.PACKSCRIPT)
+        log.debug("Pack script location is %s.", location)
+        return location
+
+    def run(self, options=None):
+        packScript = self.scriptLocation()
+        
+        arguments = []
+        arguments.append('run') # Run a script.
+        arguments.append(packScript) # The script.
+        # Argument: the number of days to leave unpacked.
+        arguments += self.conf.configData['pack_days']
+        command = ' '.join(arguments)
+        actionutils.runZopectl(self.conf, command)
+        log.info("Pack script called.")
+
+
+class ChangeOwnershipAction(Action):
+    """Action that changes ownership of some documents.
+    """
+
+    name = "Action that changes ownership of some documents."
+    option = '--changeown'
+
+    def scriptLocation(self):
+        """Return the location of the pack script.
+        """
+
+        ourDir = os.path.dirname(os.path.abspath(__file__))
+        location = os.path.join(ourDir, config.CHANGEOWNSCRIPT)
+        log.debug("Change ownership script location is %s.", location)
+        return location
+
+    def run(self, options=None):
+        changeownScript = self.scriptLocation()
+        
+        arguments = []
+        arguments.append('run') # Run a script.
+        arguments.append(changeownScript) # The script.
+        # First argument: plone site name
+        ploneSiteName = self.conf.configData['plone_site_name']
+        arguments.append(ploneSiteName)
+        # Argument: the original owner
+        arguments.append('all_powerful_Oz')
+        # Argument: the new owner
+        adminUser = self.conf.configData['user']
+        arguments.append(adminUser)
+        command = ' '.join(arguments)
+        actionutils.runZopectl(self.conf, command)
+        log.info("Change ownership script called.")
+
+
 class PrintConfigAction(Action):
     """Print the configuration.
 
@@ -487,6 +674,26 @@ class RestoreDateZopeAction(ArgumentAction):
                             fromTime=fromTime)
 
 
+class RewriteruleAction(Action):
+    """Print out a RewriteRule for use in your apache config. 
+
+    Also include some helpful pointers to information.
+    """
+
+    name = "Print out an apache rewriterule."
+    option = '--rewriterule'
+
+    def run(self, options=None):
+        ploneSiteName = self.conf.configData['plone_site_name']
+        zopePort = self.conf.configData['port']
+        sn = '%{SERVER_NAME}' # This hoses the template :-)
+        output = config.APACHE_TEMPLATE % {
+            'plonesite': ploneSiteName,
+            'zopeport': zopePort,
+            'sn': sn}
+        print output
+
+
 def getActions():
     """Return a list of action classes in this module.
     """
