@@ -53,6 +53,13 @@ parser.add_option(
     help='Only show error messages.',
     action='store_true',
     default=False)
+parser.add_option(
+    '--manifest',
+    '-m',
+    help='Print Manifest of installed Products and collisions',
+    action='store_true',
+    default=False)
+
 actions.addOptions(parser)
 
 def getProjects():
@@ -141,6 +148,33 @@ def handleMultiAction(project, multiActionName):
     for line in multiAction:
         # Turn the line into a list.
         arguments = line.split(' ')
+        log.debug("Extracting arguments from line %r.",
+                  line)
+        # Compensate for '--test "-a bcd -d efg"' arguments that are
+        # intended as one single argument.
+        copiedArguments = []
+        insideDoubleQuotes = False
+        inQuotes = []
+        for argument in arguments:
+            log.debug("Argument part: %r.", argument)
+            if insideDoubleQuotes:
+                if argument.endswith('"'):
+                    inQuotes.append(argument[:-1])
+                    joined = ' '.join(inQuotes)
+                    copiedArguments.append(joined)
+                    insideDoubleQuotes = False
+                else:
+                    inQuotes.append(argument)
+            else:
+                if argument.startswith('"'):
+                    inQuotes = [argument[1:],]
+                    insideDoubleQuotes = True
+                    log.debug("Start of stuff inside double quotes: %r",
+                              inQuotes)
+                else:
+                    copiedArguments.append(argument)
+        arguments = copiedArguments
+        log.debug("Remaining arguments: %r.", arguments)
         (options, args) = parser.parse_args(arguments)
         performActionOnProject(project, options)
 
