@@ -55,8 +55,8 @@ class BaseSource(object):
                 log.debug("%s is a .txt file, silently removing "
                           "it without warning.")
             else:
-                log.warn("'%s' exists, removing it before using %s.",
-                         target, self.name)
+                log.warn("'%s' exists, removing it before using %s on %s.",
+                         target, self.name, self.sourceName)
             # a symlink to a directory is also a directory.
             if os.path.islink(target):
                 os.remove(target)
@@ -65,7 +65,7 @@ class BaseSource(object):
             else:
                 os.remove(target)
 
-    def addProduct(self):
+    def addProduct(self, manifest=None):
         """Add a product.  
 
         This only checks if the source for that product exists.
@@ -74,10 +74,32 @@ class BaseSource(object):
         """
 
         source = self._source()
+        self.manifest=manifest
         if not os.path.exists(source):
             log.warn('Product %s not found.', source)
             raise IOError
-
+        self._addProduct()
+        
+    def _buildManifest(self, nestedSource, target):
+        manifest=self.manifest
+        if manifest is None:
+            return
+        if target.lower().endswith('.txt'):
+            return    
+        #extend the trace information - time ordered
+        trace=manifest.setdefault('trace', [])
+        try:
+            version=open(os.path.join(target, 'version.txt')).readline().strip()
+        except:
+            version=" - "
+        item=(self.name, self._source(), nestedSource, target, version)
+        trace.append(item)
+        #extend a dict of what products are installed:
+        installed=manifest.setdefault('installed', {})
+        prodname=os.path.split(target)[1]
+        sources=installed.setdefault(prodname,[])
+        sources.append(self._source() + ':' + version)
+        
 
 class SymlinkSource(BaseSource):
     """Wrapper around a symlink source.
@@ -85,14 +107,14 @@ class SymlinkSource(BaseSource):
 
     name = 'symlink'
 
-    def addProduct(self):
-        super(SymlinkSource, self).addProduct()
+    def _addProduct(self):
         productName = os.path.basename(self.sourceName)
         source = self._source()
         target = os.path.join(self.productDir, productName)
         self.removeTargetIfExists(target)
         utils.symlink(source, target, self.config)
         log.debug("Symlink added '%s'=>'%s'.", target, source)
+        self._buildManifest(None, target)
 
 
 class SymlinkbundleSource(BaseSource):
@@ -101,8 +123,7 @@ class SymlinkbundleSource(BaseSource):
 
     name = 'symlinkbundle'
 
-    def addProduct(self):
-        super(SymlinkbundleSource, self).addProduct()
+    def _addProduct(self):
         source = self._source()
         includedFiles = os.listdir(source)
         # Filter out .svn
@@ -116,6 +137,7 @@ class SymlinkbundleSource(BaseSource):
             self.removeTargetIfExists(target)
             utils.symlink(newSource, target, self.config)
             log.debug("Symlink added '%s'=>'%s'.", target, newSource)
+            self._buildManifest(newSource, target)
 
 
 class ArchiveSource(BaseSource):
@@ -130,11 +152,11 @@ class ArchiveSource(BaseSource):
             target = os.path.join(self.productDir, extractedDirectory)
             self.removeTargetIfExists(target)
             shutil.move(extractedDirectory, target)
+            self._buildManifest(None, target)
         except IndexError:
             log.warn('Product %s not found in %s', self.sourceName, tempDir)
 
-    def addProduct(self):
-        super(ArchiveSource, self).addProduct()
+    def _addProduct(self):
         source = self._source()
         # Extract zip in temp directory
         tempDir = tempfile.mkdtemp()
@@ -151,7 +173,7 @@ class ArchiveSource(BaseSource):
 
 
 class ArchivebundleSource(ArchiveSource):
-    """Wrapper around a archive bundle bundle.
+    """Wrapper around an archive bundle bundle.
     """
 
     name = 'archivebundle'
@@ -166,11 +188,13 @@ class ArchivebundleSource(ArchiveSource):
                                      extractedDirectories[0])
             os.chdir(bundleDir)
             extractedDirectories = os.listdir(bundleDir)
-            log.debug("New extracted dirs: %r.", extractedDirectories)
+            log.debug("New extracted dirs: %r.", extractedDirectories)        
         for extractedDirectory in extractedDirectories:
             target = os.path.join(self.productDir, extractedDirectory)
             self.removeTargetIfExists(target)
             shutil.move(extractedDirectory, target)
             log.debug("Moved %s.", extractedDirectory)
+            self._buildManifest(extractedDirectory, target)
+            
 
 
